export declare class ZIMLogger {
    warn(tag: string, action: string, msg?: string | Record<string, any>): void;
    /**
     * Format the log to reduce a large number of invalid logs
     *
     * 1. Uint8Array           ->  b=length
     * 2. Array.length > 2     ->  [length, Array[first], Array[last]]
     * 3. String.length > 100  ->  s=length
     * 4. ZIMMessage.message   ->  s=length
     *
     */
    private stringify;
}
export declare enum ZIMLogTag {
    Database = "DB",
    User = "User",
    Conversation = "Conv",
    Room = "Room",
    Group = "Group",
    Call = "Call",
    Friend = "Friend"
}
export declare enum ZIMLogAction {
    Login = "JSAPI.login",
    UpdateUserName = "JSAPI.updateUserName",
    UpdateUserAvatarUrl = "JSAPI.updateUserAvatarUrl",
    UpdateUserExtendedData = "JSAPI.updateUserExtendedData",
    UpdateUserOfflinePushRule = "API.updateUserOfflinePushRule",
    QueryUsersInfo = "JSAPI.queryUsersInfo",
    QuerySelfUserRule = "API.querySelfUserRule",
    QueryLocalFileCache = "API.queryLocalFileCache",
    ClearLocalFileCache = "API.clearLocalFileCache",
    SubscribeStatus = "API.subscribeUsersStatus",
    QueryStatus = "API.queryUsersStatus",
    QueryConv = "JSAPI.queryConversation",
    QueryConvList = "JSAPI.queryConversationList",
    QueryPinnedList = "JSAPI.queryConversationPinnedList",
    DeleteConv = "JSAPI.deleteConversation",
    DeleteAllConv = "JSAPI.deleteAllConversations",
    ClearUnreadCount = "JSAPI.clearConversationUnreadMessageCount",
    ClearAllUnreadCount = "JSAPI.clearConversationTotalUnreadMessageCount",
    DelAllConvMessages = "JSAPI.deleteAllConversationMessages",
    UpdatePinnedState = "JSAPI.updateConversationPinnedState",
    SetNotificationStatus = "JSAPI.setConversationNotificationStatus",
    SetConvDraft = "JSAPI.setConversationDraft",
    SetConvMark = "JSAPI.setConversationMark",
    SendMessage = "JSAPI.sendMessage",
    EditMessage = "JSAPI.editMessage",
    DeleteMessages = "JSAPI.deleteMessages",
    DeleteAllMessage = "JSAPI.deleteAllMessage",
    QueryHistoryMessage = "JSAPI.queryHistoryMessage",
    DownloadMediaFile = "JSAPI.downloadMediaFile",
    ImportLocalMessages = "JSAPI.importLocalMessages",
    ExportLocalMessages = "JSAPI.exportLocalMessages",
    InsertMessageToLocalDB = "JSAPI.insertMessageToLocalDB",
    UpdateMessageLocalExtendedData = "JSAPI.updateMessageLocalExtendedData",
    RevokeMessage = "JSAPI.RevokeMessage",
    QueryCombineMessage = "JSAPI.queryCombineMessageDetail",
    SendReceiptRead = "JSAPI.sendConversationMessageReceiptRead",
    SendMessageReceiptsRead = "JSAPI.sendMessageReceiptsRead",
    QueryReceiptsInfo = "JSAPI.queryMessageReceiptsInfo",
    QueryReceiptReadMemberList = "JSAPI.queryGroupMessageReceiptReadMemberList",
    QueryReceiptUnreadMemberList = "JSAPI.queryGroupMessageReceiptUnreadMemberList",
    AddReaction = "JSAPI.addMessageReaction",
    DeleteReaction = "JSAPI.deleteMessageReaction",
    QueryReaction = "JSAPI.queryMessageReactionUserList",
    ReplyMsg = "API.replyMessage",
    QueryReplyMsg = "API.queryMessageRepliedList",
    QueryMsgs = "API.queryMessages",
    CreateRoom = "JSAPI.createRoom",
    EnterRoom = "JSAPI.enterRoom",
    JoinRoom = "JSAPI.joinRoom",
    SwitchRoom = "JSAPI.switchRoom",
    LeaveRoom = "JSAPI.leaveRoom",
    LeaveAllRoom = "API.leaveAllRoom",
    QueryRoomMemberList = "JSAPI.queryRoomMemberList",
    QueryRoomMembers = "JSAPI.queryRoomMembers",
    QueryRoomOnlineMemberCount = "JSAPI.queryRoomOnlineMemberCount",
    SetRoomAttributes = "JSAPI.setRoomAttributes",
    DeleteRoomAttributes = "JSAPI.deleteRoomAttributes",
    QueryRoomAllAttributes = "JSAPI.queryRoomAllAttributes",
    BeginRoomAttributesBatchOperation = "JSAPI.beginRoomAttributesBatchOperation",
    EndRoomAttributesBatchOperation = "JSAPI.endRoomAttributesBatchOperation",
    SetRoomMembersAttributes = "JSAPI.setRoomMembersAttributes",
    QueryRoomMembersAttributes = "JSAPI.queryRoomMembersAttributes",
    QueryRoomMemberAttributesList = "JSAPI.queryRoomMemberAttributesList",
    CreateGroup = "JSAPI.createGroup",
    JoinGroup = "JSAPI.joinGroup",
    LeaveGroup = "JSAPI.leaveGroup",
    DismissGroup = "JSAPI.dismissGroup",
    QueryGroupList = "JSAPI.queryGroupList",
    UpdateGroupInfo = "JSAPI.updateGroupInfo",
    MuteGroup = "JSAPI.muteGroup",
    QueryGroupInfo = "JSAPI.queryGroupInfo",
    SetGroupAttributes = "JSAPI.setGroupAttributes",
    DeleteGroupAttributes = "JSAPI.deleteGroupAttributes",
    QueryGroupAttributes = "JSAPI.queryGroupAttributes",
    TransferGroupOwner = "JSAPI.transferGroupOwner",
    InviteUsersIntoGroup = "JSAPI.inviteUsersIntoGroup",
    KickGroupMembers = "JSAPI.kickGroupMembers",
    QueryGroupMemberList = "JSAPI.queryGroupMemberList",
    QueryGroupMemberCount = "JSAPI.queryGroupMemberCount",
    MuteGroupMembers = "JSAPI.muteGroupMembers",
    QueryGroupMemberMutedList = "JSAPI.queryGroupMemberMutedList",
    UpdateGroupAlias = "API.updateGroupAlias",
    SetGroupMemberNickname = "JSAPI.setGroupMemberNickname",
    SetGroupMemberRole = "JSAPI.setGroupMemberRole",
    QueryGroupMemberInfo = "JSAPI.queryGroupMemberInfo",
    UpdateGroupVerifyMode = "API.updateGroupVerifyMode",
    SendGroupJoinApp = "API.sendGroupJoinApplication",
    AcceptGroupJoinApp = "API.acceptGroupJoinApplication",
    RejectGroupJoinApp = "API.rejectGroupJoinApplication",
    SendGroupInviteApp = "API.sendGroupInviteApplications",
    AcceptGroupInviteApp = "API.acceptGroupInviteApplication",
    RejectGroupInviteApp = "API.rejectGroupInviteApplication",
    QueryGroupApp = "API.queryGroupApplicationList",
    CallInvite = "JSAPI.callInvite",
    CallCancel = "JSAPI.callCancel",
    CallAccept = "JSAPI.callAccept",
    CallReject = "JSAPI.callReject",
    CallQuit = "JSAPI.callQuit",
    CallEnd = "JSAPI.callEnd",
    CallJoin = "JSAPI.callJoin",
    CallingInvite = "JSAPI.callingInvite",
    QueryCallList = "JSAPI.queryCallInvitationList",
    AddFriend = "JSAPI.addFriend",
    SendFriendApp = "JSAPI.sendFriendApplication",
    DeleteFriends = "JSAPI.deleteFriends",
    CheckFriends = "JSAPI.checkFriendsRelation",
    UpdateFriendAlias = "JSAPI.updateFriendAlias",
    UpdateFriendAttr = "JSAPI.updateFriendAttributes",
    AcceptFriendApp = "JSAPI.acceptFriendApplication",
    RejectFriendApp = "JSAPI.rejectFriendApplication",
    QueryFriendsInfo = "JSAPI.queryFriendsInfo",
    QueryFriendList = "JSAPI.queryFriendList",
    QueryFriendAppList = "JSAPI.queryFriendApplicationList",
    AddUserToBlacklist = "JSAPI.addUsersToBlacklist",
    RemoveUsersFromBlacklist = "JSAPI.removeUsersFromBlacklist",
    CheckUserIsInBlacklist = "JSAPI.checkUserIsInBlacklist",
    QueryBlacklist = "JSAPI.queryBlacklist",
    SearchConvs = "JSAPI.searchLocalConversations",
    SearchGlobalMessages = "JSAPI.searchGlobalLocalMessages",
    SearchMessages = "JSAPI.searchLocalMessages",
    SearchGroups = "JSAPI.searchLocalGroups",
    SearchGroupMembers = "JSAPI.searchLocalGroupMembers",
    SearchFriends = "JSAPI.searchLocalFriends"
}
